Attribute VB_Name = "proVCPdriver"
' Enttec DMX Usb Pro Driver for Visual Basic. (http://www.enttec.com/)
' GPL (c) hippy (rowanmac@optusnet.com.au)

Option Explicit

' utility prototypes
Private Declare Sub CopyMemory Lib "KERNEL32" Alias "RtlMoveMemory" (lpvDest As Any, lpvSrc As Any, ByVal dwSize&)
Private Declare Sub CopyString Lib "KERNEL32" Alias "RtlMoveMemory" (lpvDest As Any, ByVal strSrc As String, ByVal dwSize&)


' Widget message types
' ====================

' firmware flashing
Private Const FLASH_FIRMWARE = 1   ' enter flash mode
Private Const FLASH_PAGE_WRITE = 2 ' flash a page
Private Const FLASH_PAGE_REPLY = 2 ' flash page success status
Private Const FLASH_PAGE_SUCCESS = "TRUE" ' recieved if last flash page write was OK
Private Const FLASH_PAGE_FAIL = "FALS" ' recieved if last flash page write failed
Private Const FLASH_PAGES = 96
Private Const FLASH_PAGE_SIZE = 64


' configuration
Public Const GET_WIDGET_CFG = 3 ' request the widgets config data
Public Const GET_WIDGET_CFG_REPLY = 3 ' returned config
Public Const SET_WIDGET_CFG = 4 ' send new config

' dmx
Public Const DMX_INPUT = 5  ' auto sent by widget apon arival of a dmx packet
Public Const DMX_OUTPUT = 6 ' sent to widget to update buffers, stay in send mode
Public Const RDM_OUTPUT = 7 ' sent to widget, send a dmx packet, then enter recieve mode
Public Const Receive_DMX_on_Change = 8 ' set send always or send on change mode

' misc packet constants
Const SOM = &H7E ' Start Of Message Marker
Const EOM = &HE7 ' End Of Message Marker


' driver error codes
' ==================
Const ERR_OK = 0               ' no error, all is good
Const ERR_WIDGET_NOT_INITIALIZED = 255 ' widget is not connected/responding
Const ERR_NOT_VALID_PACKET = 1 ' packet format seems invalid
Const ERR_PACKET_TOO_SHORT = 2 ' packet data length is not valid
Const ERR_DATA_LENGTH_INVALID = 3 ' packet specified an invalid data field length
Const ERR_FLASH_FAILED = 4 ' Failed to write last firmware page
Const ERR_DATA_LENGTH_MISMATCH = 5 ' the data length did not match the actual size in the packet


' widget structures
' =================

' message header
Type Struct_Header
 Delim As Byte ' should be equal to SOM StartOfMessage
 Label As Byte ' message type
 Length As Integer ' data field length (two bytes) 0..600
End Type
' set receive mode message header
Type Struct_RecModeHeader
 Delim As Byte ' should be equal to SOM StartOfMessage
 Label As Byte ' message type
 Length As Integer ' data field length (two bytes) 0..600
 Mode As Byte
End Type

' configuration info
Type Struct_GET_CFG
  Delim As Byte ' should be equal to SOM StartOfMessage
  Label As Byte ' message type
  Length As Integer ' data field length (two bytes) 0..600
  Firmware_VersionL As Byte
  Firmware_VersionH As Byte '
  BreakTime As Byte ' 9..127 in 10.67us units
  MarkAfterBreak As Byte ' 1..127 in 10.67us units
  FrameRate As Byte ' frames sent per second, 1..40
  UserData(0 To 507) As Byte
    
    ' user data area...
End Type

'dmx out structure
Type Struct_REQUEST_CFG
 Delim As Byte ' should be equal to SOM StartOfMessage
 Label As Byte ' message type
 Length As Integer ' data field length (two bytes) 0..600
 UserDataLength As Integer ' length of user data to return
End Type

' returned
Type Struct_SET_CFG
  Head As Struct_Header ' packet head
  DataLength As Integer ' set length of data area, two bytes, LSB first 0..508
  BreakTime As Byte ' 9..127 in 10.67us units
  MarkAfterBreak As Byte ' 1..127 in 10.67us units
  FrameRate As Byte ' frames sent per second, 1..40
End Type

' recieved dmx structure
Type Struct_RECV_DMX
  Head As Struct_Header ' packet head
  Status As Byte ' 0 = OK, otherwise Bit 0 = queue overflow, Bit 1 = overrun occured
  Data(0 To 512) As Byte  ' recieved dmx, including startcode
End Type

'dmx out structure
Type Struct_SEND_DMX
 Delim As Byte ' should be equal to SOM StartOfMessage
 Label As Byte ' message type
 Length As Integer 'Byte ' data field length (two bytes) 0..600
' LengthH As Byte ' data field length (two bytes) 0..600
 Data(0 To 512) As Byte ' dmx data, including the startcode
End Type

'firmware page structure
Type Struct_SEND_Firmware_Page
 Delim As Byte ' should be equal to SOM StartOfMessage
 Label As Byte ' message type
 Length As Integer 'Byte ' data field length (two bytes) 0..600
' LengthH As Byte ' data field length (two bytes) 0..600
 Data(0 To 63) As Byte ' firmware page data
End Type

'firmware page structure
Type Struct_Firmware_Page_Success
 Delim As Byte ' should be equal to SOM StartOfMessage
 Label As Byte ' message type
 Length As Integer 'Byte ' data field length (two bytes) 0..600
' LengthH As Byte ' data field length (two bytes) 0..600
 Data(0 To 3) As Byte ' TRUE or FALS
End Type

' contains the packet type
Public HEADER As Struct_Header
Public RecMode As Struct_RecModeHeader

' contains recieved config info
Public GET_CFG As Struct_GET_CFG
' contains setting to send to widget
Public SET_CFG As Struct_SET_CFG
' config request packet
Public REQ_CFG As Struct_REQUEST_CFG

' contains recieved dmx
Public DMX_IN As Struct_RECV_DMX
' contains dmx to send
Public DMX_OUT As Struct_SEND_DMX

' for firmware re-programming
Public FIRMWARE_PAGE As Struct_SEND_Firmware_Page
Public FIRMWARE_PAGE_REPLY As Struct_Firmware_Page_Success

' the last error to occur, see driver error constants
Public DMX_LASTERROR As Byte
Public PACKETS_RECEIVED As Long
Public LAST_TIME As Single
Public FRAMES_PER_SEC As Single

Public Sub Initialize()
' Initialize Variables
 DMX_ERROR (ERR_WIDGET_NOT_INITIALIZED)
 
 Setup_DMX_OUT_Packet (DMX_OUTPUT) ' default to send dmx mode
 Setup_CFG_Request_Packet (0) ' default to read 0 bytes user config data
 Setup_CFG_Set_Packet (0)
 PACKETS_RECEIVED = 0
End Sub
Public Sub SetReceiveMode()
 Setup_DMX_OUT_Packet (RDM_OUTPUT) ' rdm dmx mode
 LAST_TIME = Timer
End Sub

Public Sub SetSendMode()
 Setup_DMX_OUT_Packet (DMX_OUTPUT) ' send dmx mode
End Sub
Public Sub DMX_ERROR(errnum As Byte)
' declare an error has occured
 DMX_LASTERROR = errnum
End Sub
Public Function DMX_ErrorToString(errnum As Byte) As String
' returns a string description of the error number specified
Dim e As String
 Select Case errnum
   
  Case ERR_NOT_VALID_PACKET
   e = "Not Valid Packet, Recieved data seems invalid"
  Case ERR_PACKET_TOO_SHORT
   e = "Packet length is too short"
  Case ERR_DATA_LENGTH_INVALID
   e = "The packet specified an invalid data field length"
  Case ERR_FLASH_FAILED
   e = "Failed to flash firmware page"
  Case ERR_DATA_LENGTH_MISMATCH
   e = "The data length did not match the actual size in the packet"
  Case ERR_WIDGET_NOT_INITIALIZED
   e = "Waiting..."
   
  Case ERR_OK
   e = "Ready"
   
  Case Else
   e = "Unknown Error"
 End Select
 DMX_ErrorToString = e
End Function
Public Sub Set_DMX(DMXData() As Byte)
' purpose: accept a dmx array from an application
' if the data is different from that which is stored,
' then send a new packet to the widget
'
' input: a byte array of exactly 513 bytes, ie: dim dmxout(0 to 512), 0 is startcode, 1 = channel 1...
'
Dim tmp As Long

On Error GoTo setdmxerr
 
 'compare with allready buffered values
 For tmp = 0 To 512
  If DMX_OUT.Data(tmp) <> DMXData(tmp) Then tmp = 1000 ' if data is different, set tmp to 1000
 Next tmp
  
 If tmp = 512 Then Exit Sub ' data has not changed! no update required.
 
 ' copy the new dmxarray into the dmx_out structure
 CopyMemory DMX_OUT.Data(0), DMXData(0), 512
 
 Call Send_DMX_Packet ' send the new dmx packet
  
Exit Sub
setdmxerr:
 MsgBox Err.Description
End Sub

Public Function Recieve_Packet(buf As String) As Long
' input: string containing data sent from the widget (buf)
' output: if recieved dmx, copy it to DMX_IN,
'         if recieved config setting, copy them to GET_CFG
'         if recieved flash firmware page ack, send next page or fail.
Dim tmpstr As String
Dim tmpsom As Long
Dim tmpeom As Long

'On Error GoTo rxerror

Recieve_Packet = 0 ' no packet recieved..

DMX_ERROR (ERR_OK)
' check for Start of Message
For tmpsom = 1 To Len(buf)
 If Mid(buf, tmpsom, 1) = SOM Then
   buf = Mid(buf, tmpsom)
  GoTo FoundSOM
 End If
Next tmpsom
FoundSOM:
 
' check for End Of Message
For tmpeom = tmpsom To Len(buf)
 If Mid(buf, tmpeom, 1) = EOM Then
   buf = Left(buf, tmpeom)
  GoTo FoundEOM
 End If
Next tmpeom
FoundEOM:

' check packet size
 If Len(buf) < Len(HEADER) Then ' invalid packet size
  DMX_ERROR (ERR_PACKET_TOO_SHORT)
  Recieve_Packet = -1
  Exit Function
 End If

 ' read the data into a header structure..
 CopyString HEADER, buf, Len(HEADER)
 
 ' check to see if the packet delimiter is right
 If HEADER.Delim <> SOM Then ' if is not Start Of Message byte
  'MsgBox "not start of message: " & Hex(HEADER.Delim)
  DMX_ERROR (ERR_NOT_VALID_PACKET)
  Recieve_Packet = -2
  Exit Function ' fail
 End If

 ' check for silly values to help in error checking
 If HEADER.Length > 600 Then ' check this!!!
  DMX_ERROR (ERR_DATA_LENGTH_INVALID)
  Recieve_Packet = -3
  Exit Function ' fail
 End If
  
 Recieve_Packet = HEADER.Label
 Dim c As Integer
 MainForm.lblRec.Caption = "Rec: "
 For c = 1 To 10 ' Len(buf)
    MainForm.lblRec.Caption = MainForm.lblRec.Caption & Hex(Asc(Mid(buf, c, 1))) & " "
 Next c
 'frmMain.Text1 = frmMain.Text1 & vbCrLf
   ' determine what kind of packet we recieved
 Select Case HEADER.Label
  Case GET_WIDGET_CFG_REPLY ' recieved configuration data from the widget
    ' copy the data into the cfg structure
    CopyString GET_CFG, buf, (Len(buf) - 1)
    
  Case DMX_INPUT ' recieved a dmx packet from the widget
    ' copy the data from the buffer into the DMX_IN structure
    CopyString DMX_IN, buf, (Len(buf) - 1)
 End Select
 ' count number of packets received in the last second

 PACKETS_RECEIVED = PACKETS_RECEIVED + 1

 If PACKETS_RECEIVED > 1000000 Then PACKETS_RECEIVED = 0
 If Timer - LAST_TIME >= 1 Then
    FRAMES_PER_SEC = PACKETS_RECEIVED
    PACKETS_RECEIVED = 0
    LAST_TIME = Timer
 End If
 Exit Function ' success
' serious error has occured
rxerror:
 MsgBox Err.Description
End Function
Public Sub Setup_DMX_OUT_Packet(Mode As Byte)
' initialize the DMX_OUT structure, used to transmit dmx to the widget
' input: mode - either DMX_OUTPUT or RDM_OUTPUT
Dim tmp As Long
On Error GoTo packcfgerr
  If ((Mode < DMX_OUTPUT) Or (Mode > RDM_OUTPUT)) Then Mode = DMX_OUTPUT ' default to dmx output on bad input
  With DMX_OUT
   .Delim = SOM ' start of message
   .Label = Mode ' specified as DMX_OUTPUT or RDM_OUTPUT
   .Length = 513 ' sending 512 bytes 0..512
  ' .LengthH = 0
   For tmp = 0 To 512
    .Data(tmp) = 0 ' clear the data
   Next tmp
  End With
 Exit Sub ' success
packcfgerr:
 MsgBox Err.Description ' failure
End Sub

Public Sub Setup_CFG_Request_Packet(user_data_length As Integer)
' initialize the REQ_CFG structure, used to request configuration data from the widget
' input: user_data_length - how much user data to retrieve (0 = none, 508 = all)
Dim tmp As Long
On Error GoTo packcfgerr2
 If user_data_length > 508 Then user_data_length = 508
 With REQ_CFG
  .Delim = SOM
  .Label = GET_WIDGET_CFG
  .Length = 2
  .UserDataLength = user_data_length
  End With
 Exit Sub ' success
packcfgerr2:
 MsgBox Err.Description ' failure
End Sub
Public Sub Setup_CFG_Set_Packet(user_data_length As Integer)
' initialize the REQ_CFG structure, used to request configuration data from the widget
' input: user_data_length - how much user data to retrieve (0 = none, 508 = all)
Dim tmp As Long
On Error GoTo packcfgerr2
 If user_data_length > 508 Then user_data_length = 508
 With SET_CFG
  .Head.Delim = SOM
  .Head.Label = SET_WIDGET_CFG
  .Head.Length = 5
  .BreakTime = 9
  .MarkAfterBreak = 1
  .FrameRate = 40
  .DataLength = user_data_length
  End With
 Exit Sub ' success
packcfgerr2:
 MsgBox Err.Description ' failure
End Sub
Public Sub Send_Packet(buf As String)
 ' TX buf to the widget
  MainForm.MSComm1.Output = buf
  Dim c As Integer
  'frmMain.Text1 = frmMain.Text1 & "Send: "
  'For c = 1 To Len(buf)
  '   frmMain.Text1 = frmMain.Text1 & Hex(Asc(Mid(buf, c, 1))) & " "
  'Next c
  'frmMain.Text1 = frmMain.Text1 & vbCrLf
End Sub
Public Sub Send_DMX_Packet()
' purpose - sends a dmx packet to the widget from the DMX_OUT structure
Dim buf As String
Dim byBuffer(0 To Len(DMX_OUT)) As Byte

On Error GoTo senddmxerr

 CopyMemory byBuffer(0), DMX_OUT, Len(DMX_OUT)
 buf = Left$(StrConv(byBuffer, 64), Len(DMX_OUT)) & Chr(EOM) ' packet data, plus EndOfMessage marker
 
 ' now TX buf to the widget
   Send_Packet (buf)
 
 Exit Sub ' success
senddmxerr:
 MsgBox Err.Description ' fail
End Sub
Public Sub Send_CFG_Packet()
' purpose - sends a cfg packet to the widget from the SET_CFG structure, does not write user data

Dim buf As String
Dim byBuffer(0 To Len(SET_CFG)) As Byte

On Error GoTo sendcfgerr

 CopyMemory byBuffer(0), SET_CFG, Len(SET_CFG) ' copy the structure into a buffer
 buf = Left$(StrConv(byBuffer, 64), Len(SET_CFG)) & Chr(EOM)  ' convert to a string, place EndOfMessage marker
 
 ' now TX buf to the widget
   Send_Packet (buf)
  
 Exit Sub ' success
sendcfgerr:
 MsgBox Err.Description ' fail
End Sub
Public Sub Send_CFG_Request_Packet()
' purpose - sends a request config packet to the widget from the REQ_CFG structure
Dim buf As String
Dim byBuffer(0 To Len(REQ_CFG)) As Byte
On Error GoTo sendcfgreqerr

 CopyMemory byBuffer(0), REQ_CFG, Len(REQ_CFG) ' copy the structure into a buffer
 buf = Left$(StrConv(byBuffer, 64), Len(REQ_CFG)) & Chr(EOM) ' convert to a string, place EndOfMessage marker
 
 ' now TX buf to the widget
   Send_Packet (buf)
 
 Exit Sub ' success
sendcfgreqerr:
 MsgBox Err.Description ' fail
End Sub
Public Sub Send_Receive_DMX_on_Change(Mode As Integer)
 ' send receive mode setting
Dim buf As String
Dim byBuffer(0 To Len(RecMode)) As Byte

On Error GoTo sendcfgerr

 ' prepare a packet header (with no data needed) to enter into device firmware flash mode
 RecMode.Delim = SOM
 RecMode.Label = Receive_DMX_on_Change
 RecMode.Length = 1 ' one byte data
 RecMode.Mode = Mode

 CopyMemory byBuffer(0), RecMode, Len(RecMode) ' copy the structure into a buffer
 buf = Left$(StrConv(byBuffer, 64), Len(RecMode)) & Chr(EOM)  ' convert to a string, place EndOfMessage marker
 
 ' now TX buf to the widget
   Send_Packet (buf)
  
 Exit Sub ' success
sendcfgerr:
 MsgBox Err.Description ' fail
End Sub
